from __future__ import division
from collections import OrderedDict
from numpy import asarray
from matplotlib.transforms import blended_transform_factory
from matplotlib.artist import setp
from limix_util.set import OrderedSet
from . import cycler_ as cycler

class BoxPlot(object):
    def __init__(self, axes):
        self._axes = axes
        self._values = dict()
        self._colors = dict()
        self.legend = True

    def _labels(self, grp=''):
        return list(self._values[grp].keys())

    @property
    def _group_labels(self):
        return sorted(self._values.keys())

    @property
    def _unique_labels(self):
        labels = OrderedSet()
        for grp in self._group_labels:
            for l in self._labels(grp):
                labels.add(l)
        return list(labels)

    def get_color(self, label):
        return self._colors[label]

    def set_color(self, label, color):
        self._colors[label] = color

    def _get_values(self, grp):
        vs = self._values[grp].values()
        return vs

    def add(self, values, label, grp='', color=None, **props):
        if grp not in self._values:
            self._values[grp] = OrderedDict()
        self._values[grp][label] = asarray(values).ravel()

        if label not in self._colors:
            if color:
                self._colors[label] = color
            else:
                prop = cycler.next(self._axes)
                if 'color' in prop:
                    self._colors[label] = prop['color']
                else:
                    self._colors[label] = None

    def plot(self):
        axes = self._axes

        width = 0.5
        pos = []
        left = 0
        data = []
        colors = []
        for grp_label in self._group_labels:
            v = self._get_values(grp_label)
            right = left + len(v)
            pos += range(left, right)
            left = right + 1
            data.extend(v)
            colors.extend([self.get_color(l) for l in self._labels(grp_label)])

        pos = [p+1 for p in pos]
        if len(pos) == 0:
            return

        bp = axes.boxplot(data, notch=False, patch_artist=True,
                          positions=pos, widths=width)

        for i in xrange(len(colors)):
            c = colors[i]
            if c:
                setp(bp['boxes'][i], color=c, facecolor=c)
        setp(bp['whiskers'], color='black')
        setp(bp['fliers'], color='black', markerfacecolor='black', marker='+',
                           markersize=4)
        setp(bp['medians'], color='black')

        if self.legend:
            fakes = []
            for l in self._unique_labels:
                fake, = axes.plot([1,1], color=self.get_color(l), lw=3)
                fakes.append(fake)

            labels = self._unique_labels
            axes.legend(fakes, labels, bbox_to_anchor=(0., 1.02, 1., .102), loc=3,
                    ncol=min(len(labels), 4), mode="expand", borderaxespad=0.,
                    frameon=False)

            for fake in fakes:
                fake.set_visible(False)

        ngroups = len(self._values)
        import matplotlib.pylab as plt
        left = 0
        for i in xrange(ngroups):
            nboxes = len(self._labels(self._group_labels[i]))
            right = left + nboxes + 1
            center = left + (right - left) / 2
            tform = blended_transform_factory(axes.transData, axes.transAxes)
            axes.text(center, -0.03, self._group_labels[i],
                    transform=tform,
                    horizontalalignment='center', verticalalignment='top',
                    fontsize=12)
            left = right

        axes.xaxis.set_label_coords(0.5, -0.08)
        axes.set_xticklabels([''] * len(axes.get_xticklabels()))
        axes.set_xlim([pos[0] - 0.5, pos[-1] + 0.5])
        axes.yaxis.set_ticks_position('left')
        axes.xaxis.set_ticks_position('none')

        axes.spines['right'].set_visible(False)
        axes.spines['top'].set_visible(False)

        return axes
