from __future__ import division
from __future__ import absolute_import
import numpy as np

def _2Dbulk(x, left, right, y):
    xbulk = []
    ybulk = []
    for (i, xi) in enumerate(x):
        ok = np.logical_and(left <= xi, xi <= right)
        xbulk.append(xi[ok].mean())
        ybulk.append(y[i][ok].mean())

    return (np.asarray(xbulk), np.asarray(ybulk))

def _consensus_curve_same_steps(x, y, std_times):
    x = np.asarray(x)
    y = np.asarray(y)

    xavg = x.mean(0)
    yavg = y.mean(0)

    err = y.std(0) * std_times / np.sqrt(x.shape[0])

    return (xavg, yavg - err, yavg, yavg + err)

def consensus_curve(x, y, xlim=None, nparts=100, same_steps=False):
    std_times = 3

    if same_steps:
        return _consensus_curve_same_steps(x, y, std_times)

    x = np.asarray(x)
    y = np.asarray(y)

    if xlim:
        xmin, xmax = xlim
    else:
        xmin, xmax = x.min(), x.max()

    xstep = (xmax - xmin) / nparts

    xavg = []
    yavg = []

    ytop = []
    ybottom = []

    for i in range(nparts):
        left = xmin + i * xstep
        right = left + xstep

        xbulk, ybulk = _2Dbulk(x, left, right, y)

        if len(xbulk) > 0:
            xavg.append(xbulk.mean())
            yavg.append(ybulk.mean())

            err = ybulk.std() * std_times / np.sqrt(len(ybulk))
            ytop.append(yavg[-1] + err)
            ybottom.append(yavg[-1] - err)

    if xavg[0] > x.min():
        xavg.insert(0, x.min())
        yavg.insert(0, yavg[0])
        ytop.insert(0, ytop[0])
        ybottom.insert(0, ybottom[0])

    if xavg[-1] < x.max():
        xavg.append(x.max())
        yavg.append(yavg[-1])
        ytop.append(ytop[-1])
        ybottom.append(ybottom[-1])

    xavg = np.asarray(xavg)
    yavg = np.asarray(yavg)

    ytop = np.asarray(ytop)
    ybottom = np.asarray(ybottom)

    return (xavg, ybottom, yavg, ytop)

if __name__ == '__main__':
    random = np.random.RandomState(939951)
    N = 5
    x = []
    for i in range(N):
        x.append(np.linspace(0, 10, 50))

    y = []
    for i in range(N):
        y.append(np.linspace(0, 10, 50) + random.randn(50))

    xavg, ybottom, yavg, ytop = consensus_curve(x, y)
    import matplotlib.pylab as plt
    # for i in range(len(x)):
    #     plt.plot(x[i], y[i])

    plt.plot(xavg, yavg, color='red')
    plt.gca().fill_between(xavg, ybottom, ytop, lw=1.0,
                          edgecolor='black', facecolor='0.95',
                          clip_on=False)

    from limix_plot.show import show
    show(plt.gcf())
